// DragUtils.cpp: implementation of the CDragUtils class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "tinycad.h"
#include "TinyCadDoc.h"
#include "Object.h"
#include "DragUtils.h"
#include "LineUtils.h"
#include "JunctionUtils.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CDragUtils::CDragUtils(CTinyCadDoc* pDesign )
: m_j(pDesign)
{
	m_pDesign = pDesign;
	m_started = FALSE;
}

CDragUtils::~CDragUtils()
{

}

void CDragUtils::Clear()
{
	m_started = FALSE;
	m_draggedWires.erase( m_draggedWires.begin(), m_draggedWires.end() );
}


void CDragUtils::AddWireToCollection( dragWire n )
{
	// Now update this entry in the system...
	wireCollection::iterator it = m_draggedWires.find(n);
	if (it == m_draggedWires.end())
	{
		// New
		m_draggedWires.insert( n );

		// Now add any attached wires to this object...
		if (n.m_Object->GetType() == xWire || n.m_Object->GetType() == xBus)
		{
			AddAttachedObjects( n.m_Object );
		}
	}
	else
	{
		// Update
		(*it).m_MoveA |= n.m_MoveA;
		(*it).m_MoveB |= n.m_MoveB;
		if ((*it).m_Attached_LineA == NULL)
		{
			(*it).m_Attached_LineA = n.m_Attached_LineA;
			(*it).m_Distance_AlongA = n.m_Distance_AlongA;
		}
		if ((*it).m_Attached_LineB == NULL)
		{
			(*it).m_Attached_LineB = n.m_Attached_LineB;
			(*it).m_Distance_AlongB = n.m_Distance_AlongB;
		}
	}
}

// Add any wires that end at the point given to
// the drag list
void CDragUtils::AddWiresAtPoint( CPoint hs )
{
	// Find any wires attached to this pin and add them to the list
	drawingIterator it = m_pDesign->GetDrawingBegin();
	while (it != m_pDesign->GetDrawingEnd()) 
	{
		CDrawingObject *pointer = *it;

		// Only search unselected objects
		if (!m_pDesign->IsSelected( pointer ))
		{
			if ((pointer->GetType()==xWire || pointer->GetType()==xBus) 
				&& (pointer->a == hs || pointer->b==hs)) 
			{
				// Build the undo list
				m_pDesign->DupForUndo( pointer );
				AddWireToCollection( dragWire(pointer, pointer->a == hs, pointer->b == hs) );
			}
		}

		++ it;
	}
}

// Given a wire, add any objects that are attached to it...
void CDragUtils::AddAttachedObjects( CDrawingObject *wire )
{
	CLineUtils l( wire->a, wire->b );

	// Find any wires attached to this wire and add them in...
	drawingIterator it = m_pDesign->GetDrawingBegin();
	while (it != m_pDesign->GetDrawingEnd()) 
	{
		CDrawingObject *pointer = *it;

		// Is this the incoming wire?
		if (pointer == wire
			|| (single_line && pointer==m_pDesign->GetSingleSelectedItem()))
		{
			++ it;
			continue;
		}

		switch (pointer->GetType())
		{
		case xBus:
		case xWire:
			// Is this wire attached at either end to this
			// wire?
			double distance_along_a;
			double distance_along_b;
			BOOL move_a = FALSE;
			BOOL move_b = FALSE;
			
			if ( single_line || (pointer->a != wire->a
				&& pointer->a != wire->b))
			{
				move_a = l.IsPointOnLine( pointer->a, distance_along_a );
			}

			if ( single_line || (pointer->b != wire->a
				&& pointer->b != wire->b))
			{
				move_b = l.IsPointOnLine( pointer->b, distance_along_b );
			}

			// Does this object require moving?
			if (move_a || move_b)
			{
				// Build our drag wire
				dragWire n( pointer, move_a, move_b );

				if (move_a)
				{
					n.m_Distance_AlongA = distance_along_a;
					n.m_Attached_LineA = wire;
				}
				if (move_b)
				{
					n.m_Distance_AlongB = distance_along_b;
					n.m_Attached_LineB = wire;
				}
				// Build the undo list
				m_pDesign->DupForUndo( pointer );
				AddWireToCollection( n );
			}
			break;
		}
		
		++ it;
	}
}

// Move any of the objects that are attached to this object...
//
void CDragUtils::MoveAttachedObjects( CDrawingObject *wire )
{
	CLineUtils l( wire->a, wire->b );

	wireCollection::iterator it_wires = m_draggedWires.begin();
	while (it_wires != m_draggedWires.end())
	{
		BOOL Updated = FALSE;

		if (!(*it_wires).m_Done_A && (*it_wires).m_MoveA && (*it_wires).m_Attached_LineA == wire) 
		{
			Updated = TRUE;
			CPoint r = l.GetPointOnLine( (*it_wires).m_Distance_AlongA, &m_pDesign->m_snap );
			(*it_wires).m_Object->a = r;
			(*it_wires).m_Done_A = TRUE; 
		}

		if (!(*it_wires).m_Done_B && (*it_wires).m_MoveB && (*it_wires).m_Attached_LineB == wire) 
		{
			Updated = TRUE;
			CPoint r = l.GetPointOnLine( (*it_wires).m_Distance_AlongB, &m_pDesign->m_snap );
			(*it_wires).m_Object->b = r;
			(*it_wires).m_Done_B = TRUE; 
		}

		if (Updated)
		{
			(*it_wires).m_Object->Display();

			// Now move any connected wires...
			MoveAttachedObjects( (*it_wires).m_Object );
		}

		++ it_wires;
	}
}

void CDragUtils::Begin( CPoint ai, CPoint bi, BOOL single_linei )
{
	Clear();
	a = ai;
	b = bi;
	single_line = single_linei;
}

void CDragUtils::Begin()
{
	m_started = TRUE;

	// Search for Methods which might be attached to wires via pins
	m_draggedWires.erase( m_draggedWires.begin(), m_draggedWires.end() );

	selectIterator it = m_pDesign->GetSelectBegin();
	while ( it != m_pDesign->GetSelectEnd() ) 
	{
		CDrawingObject *obj=*it;
		m_pDesign->DupForUndo( obj );

		switch (obj->GetType())
		{
		case xPower:
			// Are there any wires connected to this point?
			AddWiresAtPoint( obj->a );
			break;
		case xMethodEx3:
			{
				CDrawMethod *theMethod = static_cast<CDrawMethod*>((CDrawMethod*)obj);

				CPoint tr;
				drawingCollection method;
				theMethod->ExtractSymbol( tr, method );
				m_pDesign->SetPart(theMethod->GetSubPart());
				
				// Search the symbol for pins
				drawingIterator it = method.begin();
				while (it != method.end()) 
				{
					CDrawingObject *MethodPtr = (*it);
					CDrawPin* thePin = static_cast<CDrawPin*>((CDrawPin*)MethodPtr);

					// If it is a pin then search for wires attached
					if (MethodPtr->GetType()==xPinEx && !(thePin->IsInvisible())) 
					{
						CPoint hs = thePin->GetPosition(theMethod);
						AddWiresAtPoint( hs );
					}

					++ it;
				}
			}
		}

		++ it;
	}

	// Search for wires which might need one point fixed in place
	BOOL MoveA,MoveB;
	it = m_pDesign->GetSelectBegin();
	while ( it != m_pDesign->GetSelectEnd() ) 
	{
		CDrawingObject *obj=*it;
		MoveA=FALSE;
		MoveB=FALSE;

		// Is this a wire which needs to be draged?
		if (obj->GetType()==xWire || obj->GetType()==xBus) 
		{
			if (   obj->a.x>=min(a.x,b.x) && obj->a.x<=max(a.x,b.x)
			    && obj->a.y>=min(a.y,b.y) && obj->a.y<=max(a.y,b.y) )
				MoveA = TRUE;

			if (   obj->b.x>=min(a.x,b.x) && obj->b.x<=max(a.x,b.x)
			    && obj->b.y>=min(a.y,b.y) && obj->b.y<=max(a.y,b.y) )
				MoveB = TRUE;

			// If neither point is inside this, then
			// choose to move both...
			if (!MoveA && !MoveB)
			{
				MoveA = TRUE;
				MoveB = TRUE;
				AddWiresAtPoint( obj->a );
				AddWiresAtPoint( obj->b );
			}

			AddWireToCollection( dragWire( obj, MoveA, MoveB ) );
		}

		++ it;
	}


	m_pDesign->IncUndoLevel();
}

// Call "display" on all of the wires in the dragged list
void CDragUtils::DisplayDraggedWires()
{
	wireCollection::iterator it_wires = m_draggedWires.begin();
	while (it_wires != m_draggedWires.end())
	{
		m_j.AddObjectToTodo( (*it_wires).m_Object );
		(*it_wires).m_Object->Display();
		(*it_wires).m_Done_A = FALSE;
		(*it_wires).m_Done_B = FALSE;
		++ it_wires;
	}
}


void CDragUtils::Drag( CPoint r )
{
	// Have we already started?
	if (!m_started)
	{
		Begin();
	}

	// Is there anything to do?
	if (r == CPoint(0,0))
	{
		return;
	}

	// With drag only move the points of a wire inside the update region
	selectIterator it = m_pDesign->GetSelectBegin();
	while ( it != m_pDesign->GetSelectEnd() ) 
	{
		CDrawingObject *obj=*it;

		bool defer = m_draggedWires.find(dragWire(obj)) != m_draggedWires.end();
		if (!defer)
		{
			obj->Display();

			m_j.AddObjectToTodo(obj);

			// Move the object
			obj->a += r;
			obj->b += r;

			m_j.AddObjectToTodo(obj);

			obj->Display();
		}
		
		++ it;
	}

	DisplayDraggedWires();

	wireCollection::iterator it_wires = m_draggedWires.begin();
	while (it_wires != m_draggedWires.end())
	{
		if ((*it_wires).m_MoveA && (*it_wires).m_Attached_LineA == NULL) 
		{
			(*it_wires).m_Done_A = TRUE;
			(*it_wires).m_Object->a += r;
		}

		if ((*it_wires).m_MoveB && (*it_wires).m_Attached_LineB == NULL) 
		{
			(*it_wires).m_Done_B = TRUE;
			(*it_wires).m_Object->b += r;
		}

		// Now move any attached items to this..
		MoveAttachedObjects( (*it_wires).m_Object );

		++ it_wires;
	}

	DisplayDraggedWires();

	a += r;
	b += r;

	// Now check for junctions
	m_j.CheckTodoList();
}



void CDragUtils::Move( CPoint r )
{
	// Have we already started?
	if (!m_started)
	{
		Begin();
	}

	// Is there anything to do?
	if (r == CPoint(0,0))
	{
		return;
	}

	// Move all of the objects without dragging
	selectIterator it = m_pDesign->GetSelectBegin();
	while ( it != m_pDesign->GetSelectEnd() ) 
	{
		CDrawingObject *obj=*it;
		obj->Display();

		m_j.AddObjectToTodo(obj);

		// Move the object
		obj->a += r;
		obj->b += r;

		m_j.AddObjectToTodo(obj);

		obj->Display();

		++ it;
	}

	a += r;
	b += r;

	// Now check for junctions
	m_j.CheckTodoList();
}
