/*
	TinyCAD program for schematic capture
	Copyright 1994/1995/2002,2003 Matt Pyne.

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "stdafx.h"
#include "TinyCadView.h"
#include "diag.h"
#include "colour.h"
#include "LineUtils.h"


// This defines the block edit functions

// The rectangle area slightly re-written
#define theArea CRect(a.x,a.y,b.x,b.y)







////// The block drag object //////

void CDrawBlockDrag::Paint(Context &dc,paint_options options)
{

  dc.SelectBrush();
  dc.SelectPen(PS_DOT,1,cBLOCK);
	dc.SetROP2(R2_COPYPEN);

  dc.Rectangle(theArea);
}


CDrawBlockDrag::CDrawBlockDrag(CTinyCadDoc *pDesign)
: CDrawingObject( pDesign ),
  m_drag_utils( pDesign )
{
  segment=1;
  placed=0;
  a=b=CPoint(0,0);
}

int CDrawBlockDrag::SetCursor( CPoint p )
{
	return placed ? 11 : 12;
}

void CDrawBlockDrag::EndEdit()
{
  if (!segment | placed) 
  {
	// Undo the drag
	Move(m_OldPos);

	m_pDesign->UnSelect();
	m_drag_utils.Clear();
	Display();
  }

  m_pDesign->Invalidate();
}

void CDrawBlockDrag::LButtonDown(CPoint p, CPoint s)
{
  if (placed) 
  {
	Display();
	m_pDesign->UnSelect();
	m_drag_utils.Clear();
	segment=1;
 	placed=0;
	a = b;
	m_pDesign->SetModifiedFlag( TRUE );
  } 
  else if (segment) 
  {
	a=p;
	b=p;
	segment=0;
	placed=0;
	Display();
  } 
  else 
  {
	Display();
	b=p;
	m_OldPos = p;

	// Now select all the objects in this rectangle
	m_pDesign->Select(a,b);
	if (!m_pDesign->IsSelected()) 
	{
		Display();
		RButtonDown(p,s);
		return;
	}

	placed=1;
	segment=0;

	m_drag_utils.Begin(a,b);

	// Now display the drag box
	Display();
	}
}


void CDrawBlockDrag::Move(CPoint p)
{
  if (placed) 
  {
  	CPoint r;

  	r.x=p.x-b.x;
  	r.y=p.y-b.y;

	if (r.x!=0 || r.y!=0) 
	{
		Display();
		m_drag_utils.Drag(r);

		a.x += r.x;
		a.y += r.y;
		b=p;
		Display();
	}
  } else if (!segment) {
	Display();
	b=p;
	Display();
  }

}



BOOL CDrawBlockDrag::RButtonDown(CPoint p, CPoint s)
{
  BOOL r = m_pDesign->IsSelected();

  // If necessary remove the line from the screen
  if (!segment | placed) 
  {
	// Undo the drag
	Move(m_OldPos);
	Display();
	m_pDesign->UnSelect();
	m_drag_utils.Clear();
	segment=1;
 	placed=0;
	a = b;
  }

  return r;
}




