/*
	TinyCAD program for schematic capture
	Copyright 1994/1995/2002,2003 Matt Pyne.

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "stdafx.h"
#include "TinyCadView.h"
#include "diag.h"
#include "colour.h"


// This defines the block edit functions

// The rectangle area slightly re-written
#define theArea CRect(a.x,a.y,b.x,b.y)



////// The block export object //////

void CDrawBlockExport::Paint(Context &dc,paint_options options)
{
  dc.SelectBrush();
  dc.SelectPen(PS_DOT,1,cBLOCK);
	dc.SetROP2(R2_COPYPEN);

  dc.Rectangle(theArea);
}


CDrawBlockExport::CDrawBlockExport(CTinyCadDoc *pDesign)
: CDrawingObject( pDesign )
{
  segment=1;
  a=b=CPoint(0,0);
}

void CDrawBlockExport::EndEdit()
{
  if (!segment) {
	m_pDesign->UnSelect();
	Display();
  }
}

void CDrawBlockExport::LButtonDown(CPoint p, CPoint)
{
  if (segment) {
	a=p;
	b=p;
	segment=0;
	Display();
  } else {
	Display();
	b=p;
	Display();
	// Now select all the objects in this rectangle
	m_pDesign->Select(a,b);
	if (m_pDesign->IsSelected())
		m_pDesign->Save(TRUE,TRUE);
	Display();
	segment=1;
	a = b;
	m_pDesign->UnSelect();
  }
}


void CDrawBlockExport::Move(CPoint p)
{
  if (!segment) {
	Display();
	b=p;
	Display();
  }

}

int CDrawBlockExport::SetCursor( CPoint p )
{
	return segment ? -1 : 12;
}


BOOL CDrawBlockExport::RButtonDown(CPoint p, CPoint s)
{
  BOOL r = m_pDesign->IsSelected();

  // If necessary remove the line from the screen
  if (!segment) {
	Display();
	m_pDesign->UnSelect();
	segment=1;
	a = b;
  }

  return r;
}





