/*
	TinyCAD program for schematic capture
	Copyright 1994/1995/2002,2003 Matt Pyne.

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/



// This handles the actual drawing of objects
#include "stdafx.h"
#include "TinyCadView.h"
#include "diag.h"
#include "colour.h"
#include "option.h"
#include "LineUtils.h"
#include <math.h>




////// The No Connect Class //////

#define CONNECT_SIZE 4

CDrawNoConnect::CDrawNoConnect(CTinyCadDoc *pDesign)
: CDrawingObject( pDesign )
{
	a=b=CPoint(0,0);
	segment=1;
	Undo_Level=0;
	Delete_Level=-1;
	is_stuck = FALSE;
}


void CDrawNoConnect::Load(CArchive &archive)
{
 	a = ReadPoint(archive);
  b=a;
  segment=0;
}


void CDrawNoConnect::Save(CArchive &archive)
{
  WritePoint(archive,a);
}

double CDrawNoConnect::DistanceFromPoint( CPoint p )
{
	return sqrt( (a.x-p.x)*(a.x-p.x) + (a.y-p.y)*(a.y-p.y) );
}


BOOL CDrawNoConnect::IsInside(int left,int right,int top,int bottom)
{
  return (a.x>=left && a.x<=right && a.y>=top && a.y<=bottom);
}


ObjType CDrawNoConnect::GetType()
{
  return xNoConnect;
}

CString CDrawNoConnect::GetName() const
{
  return "No Connect";
}


void CDrawNoConnect::Move(CPoint p)
{
  Display();
  BOOL is_junction;
  p = m_pDesign->GetStickyPoint(p,TRUE,FALSE,is_stuck,is_junction);
  a=p;
  b=p;
  segment=0;
  Display();
}

void CDrawNoConnect::LButtonDown(CPoint p, CPoint)
{
  Display();
  BOOL is_junction;
  p = m_pDesign->GetStickyPoint(p,TRUE,FALSE,is_stuck,is_junction);
  a=p;
  b=p;
  Store();
  Display();	// Write to screen
}

void CDrawNoConnect::Display( BOOL erase )
{
	// Invalidate the symbol
	CRect r( a.x,a.y,b.x,b.y);
	m_pDesign->InvalidateRect( r, erase, CONNECT_SIZE + 1 );

   if (is_stuck)
   {
	 r.left = a.x - HIGHLIGHT_SIZE;
	 r.right = a.x + HIGHLIGHT_SIZE;
	 r.top = a.y - HIGHLIGHT_SIZE;
	 r.bottom = a.y + HIGHLIGHT_SIZE;
	 m_pDesign->InvalidateRect( r, erase, 3 );
   } 

}



void CDrawNoConnect::Paint(Context &dc,paint_options options)
{
  switch (options)
  {
  case draw_selected:
	  dc.SelectPen(PS_SOLID,1,cSELECT);
	  break;
  case draw_selectable:
	  dc.SelectPen(PS_SOLID,1,cNOCONNECT_CLK);
	  break;
  default:
	  dc.SelectPen(PS_SOLID,1,cNOCONNECT);
  }

  
  

	dc.SetROP2(R2_COPYPEN);

  CPoint LineTo1=CPoint(a.x+CONNECT_SIZE,a.y+CONNECT_SIZE);
  CPoint LineTo2=CPoint(a.x+CONNECT_SIZE,a.y-CONNECT_SIZE);

  dc.MoveTo(CPoint(a.x-CONNECT_SIZE,a.y-CONNECT_SIZE));
  dc.LineTo(CPoint(LineTo1.x+1,LineTo1.y));
  dc.MoveTo(CPoint(a.x-CONNECT_SIZE,a.y+CONNECT_SIZE));
  dc.LineTo(CPoint(LineTo2.x+1,LineTo2.y));

  if (is_stuck)
  {
	  // Draw a nice circle to show the stickness...
	  dc.SelectBrush();
	  dc.SelectPen(PS_SOLID,5,cBUS);
	  dc.SetROP2(R2_COPYPEN);
	  dc.Ellipse(CRect(b.x+HIGHLIGHT_SIZE,b.y+HIGHLIGHT_SIZE,b.x-HIGHLIGHT_SIZE,b.y-HIGHLIGHT_SIZE));
  }
}


// Store the no connect in the drawing
CDrawingObject* CDrawNoConnect::Store()
{
  CDrawNoConnect *NewObject;

  is_stuck = FALSE;

  NewObject = new CDrawNoConnect(m_pDesign);

  *NewObject = *this;

  m_pDesign->Add(NewObject);

  m_pDesign->SetRepeat(NULL);

  return NewObject;
}



