/*
	TinyCAD program for schematic capture
	Copyright 1994/1995/2002,2003 Matt Pyne.

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/




// This handles the actual drawing of objects

#include "stdafx.h"
#include "TinyCadView.h"
#include "diag.h"
#include "colour.h"
#include "option.h"




/////// The CDrawRectOutline class /////////




// Redraw this object (including the handles)
void CDrawRectOutline::Display( BOOL erase  )
{
	CRect r( a.x,a.y,b.x,b.y);
	m_pDesign->InvalidateRect( r, erase, 8 );
}

void CDrawRectOutline::PaintHandles( Context &dc )
{
	// Put some handles around this object
	CRect r(a.x,a.y,b.x,b.y);

	CRectTracker	tracker( dc.theTransform.Scale(r), CRectTracker::dottedLine | CRectTracker::resizeOutside  );
	tracker.Draw( dc.dc );
}


// Move fields of this object about
int CDrawRectOutline::IsInsideField(CPoint p)
{
	CRect rect(a.x,a.y,b.x,b.y);
	rect.NormalizeRect();
	CRectTracker	tracker( rect, CRectTracker::dottedLine | CRectTracker::resizeOutside  );
	int r = tracker.HitTest( p );

	if (r == 8)
	{
		r = 11;
	}
	return r;
}

void CDrawRectOutline::MoveField(int w, CPoint r)
{
	Display();

	CRect rect( a.x, a.y, b.x, b.y );
	rect.NormalizeRect();

	switch (w)
	{
	case CRectTracker::hitTopLeft:
		rect.left += r.x;
		rect.top += r.y;
		break;
	case CRectTracker::hitTopRight:
		rect.right += r.x;
		rect.top += r.y;
		break;
	case CRectTracker::hitBottomRight:
		rect.right += r.x;
		rect.bottom += r.y;
		break;
	case CRectTracker::hitBottomLeft:
		rect.left += r.x;
		rect.bottom += r.y;
		break;	
	case CRectTracker::hitTop:
		rect.top += r.y;
		break;		
	case CRectTracker::hitRight:
		rect.right += r.x;
		break;		
	case CRectTracker::hitBottom:
		rect.bottom += r.y;
		break;		
	case CRectTracker::hitLeft:
		rect.left += r.x;
		break;
	case 11:
	case CRectTracker::hitMiddle:
		rect += r;
		break;		
	}

	if (a.x < b.x)
	{
		a.x = rect.left;
		b.x = rect.right;
	}
	else
	{
		b.x = rect.left;
		a.x = rect.right;
	}

	if (a.y < b.y)
	{
		a.y = rect.top;
		b.y = rect.bottom;
	}
	else
	{
		b.y = rect.top;
		a.y = rect.bottom;
	}

	Display();
}

int CDrawRectOutline::SetCursorEdit( CPoint p )
{
	return IsInsideField( p );
}


/////// The Object classes /////////




