/*
	TinyCAD program for schematic capture
	Copyright 1994/1995/2002,2003 Matt Pyne.

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include "stdafx.h"
#include <winreg.h>
#include "registry.h"

// If in Win16 mode then create our own error values
#ifdef _cWIN16_
#define	ERROR_READ_FAULT	42
#define REG_BINARY			3
#endif


////// Find out if this software is running in Win32s //////
BOOL IsWin32s()
{

#ifdef _cWIN16_
	return TRUE;
#else
	// Get the version of the operating system
	DWORD os = ::GetVersion();

	BYTE MajorVersion = static_cast<BYTE>(os & 0xff);
	BYTE MinorVersion = static_cast<BYTE>((os >> 8) & 0xff);
	WORD OsInfo = static_cast<WORD>(os >> 16);

	return (OsInfo & 0xC000) == 0x8000;
#endif

}

////// Find out if this software is running in Win95 //////
BOOL IsWin95()
{
#ifdef _cWIN16_
	return FALSE;
#else
	// Get the version of the operating system
	DWORD os = ::GetVersion();

	WORD OsInfo = static_cast<WORD>(os >> 16);

	return (OsInfo & 0xC000) == 0xC000;
#endif

}




////// Associate this software with this key //////

void Associate(const char *extension,const char *title,const char *program)
{

  // This function does not work with Win32s!
  if (IsWin32s())
  	return;

#ifndef _cWIN16_

  HKEY theKey;
  DWORD Disposition;

  // Open a key to associate the extension with a file type
  RegCreateKeyEx(HKEY_CLASSES_ROOT,extension,0,"",
	REG_OPTION_NON_VOLATILE,KEY_ALL_ACCESS,NULL,
	&theKey, &Disposition);  

  // Now set the file type of the extension
  RegSetValueEx(theKey, "" , 0, REG_SZ, (unsigned char FAR *)title, strlen(title)+1);

  RegCloseKey(theKey);

  // Write the name for this key
  RegCreateKeyEx(HKEY_CLASSES_ROOT,title,0,"",
	REG_OPTION_NON_VOLATILE,KEY_ALL_ACCESS,NULL,
	&theKey, &Disposition);  

  RegSetValueEx(theKey, "" , 0, REG_SZ, (unsigned char FAR *)title, strlen(title)+1);

  RegCloseKey(theKey);

  // Now open a key to associate the file type with this program for open key
  char KeyBuffer[256];
  strcpy(KeyBuffer,title);
  strcat(KeyBuffer,"\\shell\\open\\command");

  RegCreateKeyEx(HKEY_CLASSES_ROOT,KeyBuffer,0,"",
	REG_OPTION_NON_VOLATILE,KEY_ALL_ACCESS,NULL,
	&theKey, &Disposition);  

  strcpy(KeyBuffer,program);
  strcat(KeyBuffer," %1");
  RegSetValueEx(theKey, "" , 0, REG_SZ, (unsigned char FAR *)KeyBuffer, strlen(KeyBuffer)+1);
   
  RegCloseKey(theKey);

  // Now write the association for printing
  strcpy(KeyBuffer,title);
  strcat(KeyBuffer,"\\shell\\print\\command");

  RegCreateKeyEx(HKEY_CLASSES_ROOT,KeyBuffer,0,"",
	REG_OPTION_NON_VOLATILE,KEY_ALL_ACCESS,NULL,
	&theKey, &Disposition);  

  strcpy(KeyBuffer,program);
  strcat(KeyBuffer," /p %1");
  RegSetValueEx(theKey, "" , 0, REG_SZ, (unsigned char FAR *)KeyBuffer, strlen(KeyBuffer)+1);
   
  RegCloseKey(theKey);

#endif

}





////// The Registry Class //////



////// The Subsytem independant operations //////

void Registry::DeleteKey(const char *key)
{
  if (!Win32s)
		return;
  else {
	if (theFile == NULL)
		  return;

	fseek(theFile,0l,SEEK_SET);

	char FileName[256];
	strcpy(FileName,theFileName);
	strcat(FileName,".ini");

	// Open a tempory file to copy into
	char TempFileName[256];
	strcpy(TempFileName,theFileName);
	strcat(TempFileName,".new");
	FILE *tempFile = fopen(TempFileName,"w");

	// Scan the file for the line we are looking for
	char Buffer[512];
	char GotKey[64];
	int  GotType;

	while (!feof(theFile)) {
		char *r = fgets(Buffer,sizeof(Buffer),theFile);
		sscanf(Buffer,"%s %d",GotKey,&GotType);

		// is this the key to be deleted?
		if (strcmp(GotKey,key)!=0 && r!=NULL)
			fprintf(tempFile,"%s",Buffer);
	}

	// Close the old file
	fclose(theFile);
	fclose(tempFile);

	// Copy the temp file into the new file
	remove(FileName);
	rename(TempFileName,FileName);
	remove(TempFileName);

	// Now re-open the new file
	theFile = fopen(FileName,"r+");
  }
}



int Registry::QueryValueEx(const char *key,DWORD *Type,unsigned char FAR *data,unsigned long *size)
{
  if (!Win32s) {
#ifndef _cWIN16_
	return RegQueryValueEx( theKey, key, NULL, Type, (unsigned char *)data, size);
#endif
  } else {
  	// Read the file
	if (theFile == NULL)
		  return ERROR_READ_FAULT;

	fseek(theFile,0l,SEEK_SET);

	// Scan the file for the line we are looking for
	char Buffer[4192];
	char GotKey[256];
	int  GotType;
	while (!feof(theFile)) {
		fgets(Buffer,sizeof(Buffer),theFile);
		sscanf(Buffer,"%s %d",GotKey,&GotType);
		if (strcmp(GotKey,key)==0) {
			// Remove the trailing CR
			Buffer[strlen(Buffer)-1] = 0;
			
			// Find the data string from the buffer
			int p = 0,c = 0;
			while (Buffer[p]!=0 && c!=2) {
				if (Buffer[p++] == ' ')
					c++;
			}

			*Type = GotType;
			if (GotType == REG_SZ) {
				#ifdef _cWIN16_
				for (*size = 0;Buffer[*size + p] !=0 && *size<1024;(*size)++)
					data[*size] = Buffer[*size + p];
				data[*size] = 0;
				#else
				strcpy((char *)data,Buffer+p);
				*size = strlen(Buffer+p);
				#endif
			} else {
				int OutCount = 0;
				for (int Count =p; Buffer[Count] != 0 && Buffer[Count+1] != 0; Count+= 2) {
					#define HexNibble(a) ((a>='a') ? (a - 'a'+10) : (a - '0'))
					data[OutCount++] = HexNibble(Buffer[Count]) << 4 | HexNibble(Buffer[Count+1]);
				}
				*size = OutCount;
			}
			return ERROR_SUCCESS;
		}
	}

  return ERROR_READ_FAULT;
  }

  return ERROR_READ_FAULT;
}



int Registry::SetValueEx(const char *key, DWORD Type, unsigned char FAR *data, unsigned long length)
{
  if (!Win32s) {
#ifndef _cWIN16_
	return RegSetValueEx(theKey, key, 0, Type, data, length);
#endif
  } else {

  	// Write to file
	if (theFile == NULL)
		  return ERROR_READ_FAULT;

	// Delete the key in the file first
	DeleteKey(key);

	// Now add it to the end of the file
	fseek(theFile,0l,SEEK_END);
	fprintf(theFile,"%s %d ",key,Type);

	if (Type != REG_SZ) {
		for (unsigned long count = 0; count != length; count++)
			fprintf(theFile,"%02x",data[count]);
		fprintf(theFile,"\n");
	} else
		fprintf(theFile,"%s\n",data);

	return ERROR_SUCCESS;
  }

  return ERROR_READ_FAULT;
}





// Attach this object to the registry
Registry::Registry()
{
  Win32s = IsWin32s();

  // Open the key associated with this name
  if (!Win32s) {
#ifndef _cWIN16_
    char KeyBuffer[256];
    strcpy(KeyBuffer,"Software\\TinyCAD\\");
	strcat(KeyBuffer,NAME"\\"REG_REVISION);

	DWORD Disposition;
  	RegCreateKeyEx(HKEY_CURRENT_USER,KeyBuffer,0,"",
		REG_OPTION_NON_VOLATILE,KEY_ALL_ACCESS,NULL,
		&theKey, &Disposition);
#endif
  } else {
  	// Open the file associated with this program
	char FileName[256];
	GetWindowsDirectory(FileName,sizeof(FileName));
	strcat(FileName,"\\"NAME);
	strcpy(theFileName,FileName);
	strcat(FileName,".ini");
	theFile = fopen(FileName,"r+");
	if (theFile == NULL)
		theFile = fopen(FileName,"w+");
  }
}

CString Registry::getKey()
{
    char KeyBuffer[256];
    strcpy(KeyBuffer,"Software\\TinyCAD\\");
	strcat(KeyBuffer,NAME"\\"REG_REVISION);

	return KeyBuffer;
}


// Close the registry database
Registry::~Registry()
{
  if (!Win32s) {
#ifndef _cWIN16_
  	RegCloseKey(theKey);
#endif
  } else {
  	fclose(theFile);
  }
}








// Place an item into the registry
void Registry::Place(const char *key,const char FAR *data)
{
	int Length;
#ifdef _cWIN16_
	for (Length = 0; data[Length] != 0 && Length < 1024; Length ++) ;
#else
	Length = strlen(data);
#endif
	SetValueEx(key, REG_SZ, (unsigned char FAR *)data, Length+1);
}


// Place an item into the registry
void Registry::Place(const char *key,WORD data)
{
	SetValueEx(key, REG_BINARY, (unsigned char FAR *)&data, sizeof(WORD));
}


// Place an item into the registry
void Registry::Place(const char *key,void FAR *data,int length)
{
	SetValueEx(key, REG_BINARY, (unsigned char FAR *)((char FAR *)data), length);
}


// Get a sub item from the registry
BOOL Registry::Get(const char *key,char FAR *data,unsigned int length)
{
  DWORD type;
  DWORD size=length;
  int r;

  r = QueryValueEx( key, &type,(unsigned char FAR *)data, &size);

  return r == ERROR_SUCCESS;
}


BOOL Registry::Get(const char *key, CString &data)
{
  DWORD type;
  DWORD size=1024;
  int r;

  r = QueryValueEx( key, &type, (unsigned char *)(data.GetBuffer(size)), &size);
  data.ReleaseBuffer();

  return r == ERROR_SUCCESS;
}


BOOL Registry::Get(const char *key,void *data,unsigned int length)
{
  DWORD type;
  DWORD size=length;
  int r;

  r = QueryValueEx( key, &type,(unsigned char *)data, &size);

  if ( r == ERROR_SUCCESS && size!=length )
	realloc(data,size);

  return r == ERROR_SUCCESS;
}


BOOL Registry::Get(const char *key,WORD *data)
{
  DWORD type;
  DWORD size=sizeof(WORD);
  int r;

  r = QueryValueEx( key, &type,(unsigned char *)data, &size);

  return r == ERROR_SUCCESS;
}



BOOL Registry::hGet(const char *key,HANDLE hdata,unsigned int length)
{
  DWORD type;
  DWORD size=length;
  int r;
  
  void FAR *data;
  
  data = ::GlobalLock(hdata);

  r = QueryValueEx( key, &type,(unsigned char FAR *)data, &size);

  ::GlobalUnlock(hdata);

  return r == ERROR_SUCCESS;
}


////// These functions use the registry //////

void Registry::SavePrinterMode(HANDLE DevMode,HANDLE PrinterName)
{
	DEVMODE  FAR *theDevMode = (DEVMODE FAR *)::GlobalLock(DevMode);
	DEVNAMES FAR *thePrinterName = (DEVNAMES FAR *)::GlobalLock(PrinterName);

  	if (theDevMode != NULL)
		Place("PrinterMode",(void FAR *)theDevMode,theDevMode->dmSize+theDevMode->dmDriverExtra);

	if (thePrinterName != NULL) {
		CString theNames;
		theNames = 	((char FAR *)thePrinterName)+thePrinterName->wDriverOffset;
		theNames += ",";
		theNames += ((char FAR *)thePrinterName)+thePrinterName->wDeviceOffset;
		theNames += ",";
		theNames += ((char FAR *)thePrinterName)+thePrinterName->wOutputOffset;
		
		Place("Printer",theNames);
	}

	::GlobalUnlock(DevMode);
	::GlobalUnlock(PrinterName);
}


void Registry::SavePageSize(CPoint tbePageSize)
{
  char String[256];

  _itoa(tbePageSize.x,String,10);
  strcat(String,",");
  _itoa(tbePageSize.y,String+strlen(String),10);
  Place("PageSize",String);
}




////// The find files class //////


// Find the first file

FindFile::FindFile(CString FileSpec)
{
#ifdef _cWIN16_
  handle = _dos_findfirst(FileSpec.GetBuffer(1024), _A_NORMAL, &fileinfo );  
  FileSpec.ReleaseBuffer();
#else
  handle = ::FindFirstFile(FileSpec, &fileinfo );
#endif
}


// Find the next file
BOOL FindFile::FindNext()
{
#ifdef _cWIN16_
  return _dos_findnext(&fileinfo) == 0;
#else
  return ::FindNextFile(handle,&fileinfo);
#endif
}

// Close the FindFile structure
FindFile::~FindFile()
{
#ifdef _cWIN16_
#else
	::FindClose(handle);	
#endif
}


char *FindFile::GetName()
{
#ifdef _cWIN16_
	return fileinfo.name;
#else
	return fileinfo.cFileName;	
#endif

}


// Was the find file successfull?
BOOL FindFile::Success()
{

#ifdef _cWIN16_
	return handle == 0;
#else
	return handle != INVALID_HANDLE_VALUE;
#endif
	
}
