/*
	TinyCAD program for schematic capture
	Copyright 1994/1995/2002 Matt Pyne.

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#if !defined(AFX_TINYCADDOC_H__34C3ADE5_C40F_411E_B21B_E123312BD3BE__INCLUDED_)
#define AFX_TINYCADDOC_H__34C3ADE5_C40F_411E_B21B_E123312BD3BE__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "context.h"
#include "object.h"
#include "Library.h"

class CLibraryFile;
class CTinyCadView;

#include "DlgDetailsBox.h"

// The update hints
#define		DOC_UPDATE_INVALIDATE		100
#define		DOC_UPDATE_INVALIDRECTERASE	101
#define		DOC_UPDATE_INVALIDRECT		102
#define		DOC_UPDATE_SETCURSOR		103

struct doc_invalidrect : public CObject
{
	CRect r;
	int grow;
};

typedef std::set<CDrawingObject*> selectCollection;
typedef selectCollection::iterator selectIterator;


class CTinyCadDoc : public CDocument
{
protected: // create from serialization only
	DECLARE_DYNCREATE(CTinyCadDoc)

// Attributes
public:

// Operations
public:

// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CTinyCadDoc)
	public:
	virtual BOOL OnNewDocument();
	virtual void Serialize(CArchive& ar);
	virtual BOOL OnOpenDocument(LPCTSTR lpszPathName);
	//}}AFX_VIRTUAL

// Implementation
public:
	CTinyCadDoc();
	virtual ~CTinyCadDoc();
#ifdef _DEBUG
	virtual void AssertValid() const;
	virtual void Dump(CDumpContext& dc) const;
#endif

protected:

	friend	CDlgDetailsBox;
	friend	CDrawingObject;


	/////////////////////////////////////////////////////////////////////////////
	//

	// The current object being used to draw on this object
	//

	CDrawingObject	*edit;			// The object currently being edited

	CDrawingObject 	*NextRepeat;	// The repeat object (NULL if none)
	CDrawingObject 	*LastRepeat;	// The last repeated object (NULL if none)
	CPoint	RepeatDir;				// The direction to move when repeating

	int 	NameDir;				// +ve/-ve name increments when repeating
	int 	PinDir;					// +ve/-ve pin number increments when repeating

	CDrawingObject	*selectable;	// The object that will be selected when the user clicks


	/////////////////////////////////////////////////////////////////////////////
	// Create the options
	//
	Option theOptions;


	/////////////////////////////////////////////////////////////////////////////
	//
	int		part;				// The currently displayed part

	/////////////////////////////////////////////////////////////////////////////
	//

	int MaxXSize;					// The size of the page
	int MaxYSize;

	int Undo_Level;					// The current undo level
	int Top_Level;					// The highest undo level in the list
	int Bottom_Level;				// The lowest possible undo level
	BOOL Undo_Inc;  				// Flag to increment on next add

	CString DateStr;				// The date when this design was last edited/saved
  	CString	Title;					// The title of this design
	CString Author;					// The author of this design
	CString Revision;				// The revision code of this design
	CString DocNumber;				// The document number of this design
	CString Organisation;			// The organisation which designed this design
	CString Sheets;					// The number of sheets in this design
	BOOL	ShowDetails;			// Do we display the details box?
	BOOL	ShowRulers;				// Do we show the rulers at the side of the page?

protected:

	drawingCollection	m_drawing;
	selectCollection	m_selected;

	drawingCollection	m_redo;

public:

	/////////////////////////////////////////////////////////////////////////////
	//
	// Drawing collection operators
	//

	// Iterate through the drawing
	drawingIterator GetDrawingBegin();
	drawingIterator GetDrawingEnd();
	
	// Add an object to this drawing
	void Add( CDrawingObject * );
	void Erase( drawingIterator it );
	

	
	/////////////////////////////////////////////////////////////////////////////
	//
	// Selection operators
	//

	// Iterate through the selected objects
	selectIterator GetSelectBegin();
	selectIterator GetSelectEnd();
	BOOL IsSelected( CDrawingObject * p );
	BOOL IsSingleItemSelected();
	BOOL IsSelected(); 
	CDrawingObject* GetSingleSelectedItem();


	// Select an area
	void Select(CPoint,CPoint);

	// Select an object
	void Select(CDrawingObject *);

	// Unselect all objects
	void UnSelect();

	// Unselect a single object
	void UnSelect( CDrawingObject *p );

	// Delete the selected objects
	void SelectDelete();

	// Move the selected objects
	void SelectMove(CPoint);

	// Duplicate the selected objects
	void SelectDup();

	// Undo the move of the selected objects
	void SelectUndoMove(CPoint);

	// Change the Z-order of the selected objects
	void BringToFront();
	void SendToBack();


	CDrawingObject* GetSelectable();
	void SetSelectable( CDrawingObject *obj );

	// The current grid settings
	TransformSnap	m_snap;

	// Is this document editing a library?
	virtual bool IsLibInUse( CLibraryFile *lib );

	void SetPageBoundries(PRINTDLG &);		// Set the page boundries from a print dialog
	void SetPageBoundries(CPoint NewSize) { MaxXSize=NewSize.x; MaxYSize=NewSize.y; }
											// Set the page boundries from a CPoint
	CPoint GetPageBoundries() { return CPoint(MaxXSize,MaxYSize); }
											// Return the page boundries in a CPoint
	BOOL IsPortrait() { return MaxXSize < MaxYSize; }
	void Undo();
	void Redo();
	int GetUndoLevel() { return Undo_Level; }
	void IncUndoLevel() { Undo_Inc = TRUE; }
	void SwapLists();
	void FlushRedo();						// Remove the Redo Buffer

	void Initalize(CTinyCadView *window);

	void DisplayRulers(Context &dc);	// Draw the design rulers
	void DisplayDetails(Context &);		// Draw the design details box

	BOOL Import();
	BOOL Import( CArchive& );
	BOOL ReadFile(CArchive &theArchive, BOOL Details, drawingCollection &drawing);
	BOOL Save(BOOL = TRUE, BOOL = FALSE);
	void Save(CArchive&, drawingCollection &drawing, BOOL Details = FALSE, BOOL SaveSelect = FALSE, BOOL SaveResources = TRUE);
	void Save(CArchive&, BOOL Details = FALSE, BOOL SaveSelect = FALSE);

	void ResetSymbols();
	void DeleteErrors();
	void TagAllResources();

	void DupForUndo( CDrawingObject *pObject );

	BOOL CanUndo() { return (Undo_Level!=Top_Level); }
	BOOL CanRedo() { return (Undo_Level!=Bottom_Level); }

	// Redraw our window
	void Invalidate();
	void InvalidateRect( CRect r, BOOL erase, int grow, BOOL outline_only = FALSE );
	void ForceSetCursor();


	// Set the next repeat object for the editing view
	void SetRepeat( CDrawingObject *pObject );
	CPoint GetRepeatDir() { return CPoint(RepeatDir.x * m_snap.GetGrid(), RepeatDir.y * m_snap.GetGrid()); }
	int GetNameDir() { return NameDir; }
	int GetPinDir() { return PinDir; }
	void SetRepeatDir(int);	// Change the repeat direction
	void SetNameDir( int i ) { NameDir = i; }
	void SetPinDir( int i ) { PinDir = i; }
	void Repeat();
	BOOL CanRepeat() { return NextRepeat != NULL; }

	// Snap to grid on/oof
	BOOL GetSnapToGrid() { return m_snap.GetGridSnap(); }
	void SetSnapToGrid( BOOL r ) { m_snap.SetGridSnap(r); }

	// Select the editing object
	void SelectObject(CDrawingObject *NewO);

	// Get the current edit object
	CDrawingObject *GetEdit() { return edit; }

	// Edit a library symbol
	virtual BOOL IsEditLibrary() { return FALSE; }

	// Get/set the current part number
	void SetPart( int i ) 
	{ 
		part = i; 
	}

	int GetPart() 
	{ 
		return part; 
	}

	// Get the nearest point that matches that attaches
	// to a schema point...
	CPoint GetStickyPoint( CPoint q, BOOL pins, BOOL wires, BOOL &is_stuck, BOOL &is_junction );

	// Add a new junction to the drawing at the specified point
	void AddNewJunction( CPoint q );

	// No obvious answer to this one, but it is
	// a function that should go in here...
	CSize GetTextExtent(const char *str, int style);

	// Get the current options for this document
	Option *GetOptions() { return &theOptions; }

	// Create an enhanced metafile from this function
	HENHMETAFILE CreateMetafile( CDC &ref_dc );

	// Called after a paste or import to enable the
	// document to sort out the imported block when
	// necessary
	virtual void PostPaste();

// Generated message map functions
protected:
	//{{AFX_MSG(CTinyCadDoc)
		// NOTE - the ClassWizard will add and remove member functions here.
		//    DO NOT EDIT what you see in these blocks of generated code !
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};

/////////////////////////////////////////////////////////////////////////////

// Some function associated with designs but not actually part of the class
void HeaderStamp(CArchive &);
BOOL CheckHeaderStamp(CArchive &);


//{{AFX_INSERT_LOCATION}}
// Microsoft Visual C++ will insert additional declarations immediately before the previous line.

#endif // !defined(AFX_TINYCADDOC_H__34C3ADE5_C40F_411E_B21B_E123312BD3BE__INCLUDED_)
