/*
	TinyCAD program for schematic capture
	Copyright 1994/1995/2002 Matt Pyne.

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/



// TinyCadSymbolDoc.cpp: implementation of the CTinyCadSymbolDoc class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "tinycad.h"
#include "TinyCadSymbolDoc.h"
#include "diag.h"
#include "library.h"
#include "Object.h"
#include "DlgUpdateBox.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CTinyCadSymbolDoc::CTinyCadSymbolDoc()
{
  libedit = NULL;
}

CTinyCadSymbolDoc::~CTinyCadSymbolDoc()
{

}

/////////////////////////////////////////////////////////////////////////////
// CTinyCadSymbolDoc

IMPLEMENT_DYNCREATE(CTinyCadSymbolDoc, CTinyCadDoc)

BEGIN_MESSAGE_MAP(CTinyCadSymbolDoc, CTinyCadDoc)
	//{{AFX_MSG_MAP(CTinyCadSymbolDoc)
	ON_COMMAND(ID_LIBRARY_ADDPIN, OnLibraryAddpin)
	ON_COMMAND(ID_LIBRARY_SELECTPARTINPACKAGE, OnLibrarySelectpartinpackage)
	ON_COMMAND(ID_FILE_SAVE, OnFileSave)
	ON_COMMAND(ID_FILE_SAVE_AS, OnFileSaveAs)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()




void CTinyCadSymbolDoc::setSymbol(CLibraryFile *library, CLibraryFileSymbol &symbol)
{
	// Store the information
	symboledit = symbol;
	libedit = library;

	// Set our name according to the symbol
	CString title = "Symbol - " + symbol.name;
	SetTitle( title );

  // Now load the symbol for editing ...
	CPoint tr;

	CDesignFileSymbol *pSymbol = symbol.GetDesignSymbol( this );
	drawingCollection method;
	if (pSymbol)
	{
		pSymbol->GetMethod( tr,method);
	}

	UnSelect();

  drawingIterator it = method.begin();
  while (it != method.end()) 
  {
	  CDrawingObject *pointer = *it;
	CDrawingObject *obj = pointer->Store();
	Select(obj);

	++ it;
  }

  tr = CPoint(-tr.x + 100, -tr.y + 100 );
  SelectMove( tr );
  UnSelect();

  // Reset the part number to zero
  SetPart(0);
  
  // Now place the symbol where it is required
  delete pSymbol;

}

// Is this document editing a library?
bool CTinyCadSymbolDoc::IsLibInUse( CLibraryFile *lib )
{
	return lib == libedit;
}


void CTinyCadSymbolDoc::OnLibraryAddpin() 
{
	SelectObject(new CDrawPin(this));
}

void CTinyCadSymbolDoc::OnLibrarySelectpartinpackage() 
{
  int OldPart = GetPart();
  CDlgWhichDevice theDialog(GetPart(),AfxGetMainWnd());

  if (theDialog.DoModal()==IDOK) {
	
	// Get rid of any drawing tool
	SelectObject(new CDrawEditItem(this));

	SetPart(theDialog.GetPart());
	// Are there any pins selected for this part
	int innew = FALSE,inold = FALSE;

	drawingIterator it = GetDrawingBegin();
	while (it != GetDrawingEnd()) 
	{
		CDrawingObject *pointer = *it;

		if (pointer->GetType()==xPinEx && ((CDrawPin *)pointer)->GetPart()==GetPart())
				innew = TRUE;
		if (pointer->GetType()==xPinEx && ((CDrawPin *)pointer)->GetPart()==OldPart)
				inold = TRUE;
		
		++ it;
	}
	// Do we need to copy over the pins?
	if (!innew && inold && Message(IDS_COPYPINS,MB_YESNO | MB_ICONQUESTION)==IDYES) 
	{
		CDrawPin *NewPin;
		drawingIterator it = GetDrawingBegin();
		while (it != GetDrawingEnd()) 
		{
			CDrawingObject *pointer = *it;
			if (pointer->GetType()==xPinEx && ((CDrawPin *)pointer)->GetPart()==OldPart) {
				NewPin = new CDrawPin(this);
				*NewPin = *((CDrawPin *)pointer);
				NewPin->SetPart(GetPart());
				Add(NewPin);
			}
			
			++ it;
		}
	}

	Invalidate();
  }	
}

void CTinyCadSymbolDoc::OnFileSave() 
{
	Store();
}

void CTinyCadSymbolDoc::OnFileSaveAs() 
{
	Store();
}


BOOL CTinyCadSymbolDoc::CanCloseFrame(CFrameWnd* pFrameArg)
{
	CString prompt;
	AfxFormatString1(prompt, AFX_IDP_ASK_TO_SAVE, symboledit.name);
	switch (AfxMessageBox(prompt, MB_YESNOCANCEL, AFX_IDP_ASK_TO_SAVE))
	{
	case IDCANCEL:
		return FALSE;
	case IDYES:
		return Store();
	case IDNO:
		break;
	}

	return TRUE;
}

// Write this symbol back to the library
BOOL CTinyCadSymbolDoc::Store()
{
	// Find out how many parts in this package
	int max=0;
	drawingIterator it = GetDrawingBegin();
	while (it != GetDrawingEnd()) 
	{
		CDrawingObject *pointer = *it;

		if (pointer->GetType()==xPinEx && ((CDrawPin *)pointer)->GetPart()>max)
		{
			max = ((CDrawPin *)pointer)->GetPart();
		}

		++ it;
	}
	symboledit.ppp = max+1;

	CDlgUpdateBox dlg( AfxGetMainWnd() );
	dlg.SetSymbol( &symboledit );

	if (dlg.DoModal() == IDOK)
	{
		theOptions.UnTag();
		libedit->Store( *this );
		return TRUE;
	}
	else
	{
		return FALSE;
	}

}



// Called after a paste or import to enable the
// document to sort out the imported block when
// necessary
void CTinyCadSymbolDoc::PostPaste()
{
	// Scan and convert any imported symbols
	// into their component parts
	drawingIterator it = GetDrawingBegin();
	while (it != GetDrawingEnd()) 
	{
		drawingIterator current = it;
		++ it;
		CDrawingObject *pObject = *current;

		// Is this a method object?
		if (   pObject->GetType() == xMethodEx3
			&& IsSelected(pObject))
		{
			// Convert to the actual type
			CDrawMethod *pMethod = static_cast<CDrawMethod*>(pObject);

			// Remove the method from the linked list
			Erase( current );

			// Get the symbol data
			CPoint tr;
			drawingCollection method;
			pMethod->ExtractSymbol( tr,method );
			
			// Now re-insert using the offset of the main
			// method
			CPoint offset = method.front()->a;
			drawingIterator it = method.begin();
			while (it != method.end())
			{
				CDrawingObject *pInsertObject = *it;
				CDrawingObject *pDup = pInsertObject->Store();
				
				pDup->a += offset;
				pDup->b += offset;

				Select( pDup );
			}

			// ... and delete the method
			UnSelect(pMethod);
			delete pMethod;
		}
	}
}
