/*
	TinyCAD program for schematic capture
	Copyright 1994/1995/2002,2003 Matt Pyne.

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "stdafx.h"
#include "option.h"
#include "revision.h"
#include "registry.h"
#include "TinyCadView.h"
#include "TinyCad.h"
#include "MainFrm.h"
#include "diag.h"





void CEditDlg::Show( CTinyCadDoc *pDesign, CDrawingObject *pObject )
{
	m_pDesign = pDesign;
	m_pObject = pObject;

  if (++opens==1) 
  {
	  g_EditToolBar.setDlg( this );
  }

}

void CEditDlg::Close()
{
  if (--opens==0)
  {
	  g_EditToolBar.unsetDlg();
	  ShowWindow( SW_HIDE );
  }
}


CDrawingObject *CEditDlg::getObject()
{
	
	return m_pObject;
}


void CEditDlg::changeSelected( CDrawingObject *previous, CDrawingObject *pObject )
{
	if (previous == m_pObject)
	{
		m_pObject = pObject;
	}
}

////// The no tool dialog box //////

void CEditDlgNoTool::Create()
{
	CDialog::Create( IDD_NO_TOOL, &g_EditToolBar );
}



////// The Power editing Dialog //////

BEGIN_MESSAGE_MAP( CEditDlgPowerEdit, CEditDlg )
	ON_EN_CHANGE(POWERBOX_TEXT, OnChange)
	ON_BN_CLICKED(POWERBOX_TOP, OnChange)
	ON_BN_CLICKED(POWERBOX_BOTTOM, OnChange)
	ON_BN_CLICKED(POWERBOX_LEFT, OnChange)
	ON_BN_CLICKED(POWERBOX_RIGHT, OnChange)
	ON_BN_CLICKED(POWERBOX_WAVE, OnChange)
	ON_BN_CLICKED(POWERBOX_CIRCLE, OnChange)
	ON_BN_CLICKED(POWERBOX_ARROW, OnChange)
	ON_BN_CLICKED(POWERBOX_BAR, OnChange)
	ON_BN_CLICKED(POWERBOX_EARTH, OnChange)
END_MESSAGE_MAP()

// Create this window
void CEditDlgPowerEdit::Create()
{
	  CDialog::Create( IDD_POWER, &g_EditToolBar );
}

void CEditDlgPowerEdit::Open(CTinyCadDoc *pDesign, CDrawingObject *pObject)
{
  stop=TRUE;
  
	Show( pDesign, pObject );
  SetDlgItemText(POWERBOX_TEXT,static_cast<CDrawPower*>(getObject())->str);
  CheckRadioButton(POWERBOX_TOP,POWERBOX_RIGHT,(static_cast<CDrawPower*>(getObject())->dir)+POWERBOX_TOP);
  CheckRadioButton(POWERBOX_BAR,POWERBOX_EARTH,(static_cast<CDrawPower*>(getObject())->which)+POWERBOX_BAR);

  stop=FALSE;
}


void CEditDlgPowerEdit::OnChange()
{
  char str[SIZESTRING];
  if (stop)
	return;

  getObject()->Display();

  GetDlgItemText(POWERBOX_TEXT, str, sizeof(str) );
  static_cast<CDrawPower*>(getObject())->str = str;

  static_cast<CDrawPower*>(getObject())->dir=GetCheckedRadioButton(POWERBOX_TOP,POWERBOX_RIGHT)-POWERBOX_TOP;
  static_cast<CDrawPower*>(getObject())->which=GetCheckedRadioButton(POWERBOX_BAR,POWERBOX_EARTH)-POWERBOX_BAR;

  getObject()->Display();
}




// The Pin editing Dialog

BEGIN_MESSAGE_MAP( CEditDlgPinEdit, CEditDlg )
	ON_EN_CHANGE(PINBOX_NAME, OnChange)
	ON_EN_CHANGE(PINBOX_NUMBER, OnChange)
	ON_EN_CHANGE(PINBOX_LENGTH, OnChange)
	ON_BN_CLICKED(PINBOX_NAMEON, OnChange)
	ON_BN_CLICKED(PINBOX_NUMBERON, OnChange)
	ON_BN_CLICKED(PINBOX_UP, OnChange)
	ON_BN_CLICKED(PINBOX_DOWN, OnChange)
	ON_BN_CLICKED(PINBOX_LEFT, OnChange)
	ON_BN_CLICKED(PINBOX_RIGHT, OnChange)
	ON_BN_CLICKED(PINBOX_NORMAL, OnChange)
	ON_BN_CLICKED(PINBOX_DOT, OnChange)
	ON_BN_CLICKED(PINBOX_CLOCK, OnChange)
	ON_BN_CLICKED(PINBOX_DCLOCK, OnChange)
	ON_BN_CLICKED(PINBOX_POWER, OnChange)
	ON_BN_CLICKED(PINBOX_HIDDEN, OnChange)
	ON_BN_CLICKED(PINBOX_INPUT, OnChange)
	ON_BN_CLICKED(PINBOX_OUTPUT, OnChange)
	ON_BN_CLICKED(PINBOX_BIDIR, OnChange)
	ON_BN_CLICKED(PINBOX_PASSIVE, OnChange)
	ON_BN_CLICKED(PINBOX_TRISTATE, OnChange)
END_MESSAGE_MAP()

CEditDlgPinEdit::CEditDlgPinEdit()
{
  opens=0;
  stop=TRUE;
}


// Create this window
void CEditDlgPinEdit::Create()
{
	  CDialog::Create( IDD_PIN, &g_EditToolBar );
}

void CEditDlgPinEdit::Open(CTinyCadDoc *pDesign, CDrawingObject *pObject)
{
  stop=TRUE;
  
  Show( pDesign, pObject );

  char Buffer[2];
  Buffer[0] = static_cast<CDrawPin*>(getObject())->part+'A';
  Buffer[1] = 0;

  SetDlgItemText(PINBOX_NAME,static_cast<CDrawPin*>(getObject())->str);
  SetDlgItemText(PINBOX_NUMBER,static_cast<CDrawPin*>(getObject())->number);
  SetDlgItemText(PINBOX_PART,Buffer);
  SetDlgItemInt(PINBOX_LENGTH,static_cast<CDrawPin*>(getObject())->length);
  CheckRadioButton(PINBOX_UP,PINBOX_RIGHT,static_cast<CDrawPin*>(getObject())->dir+PINBOX_UP);
  CheckRadioButton(PINBOX_NORMAL,PINBOX_HIDDEN,static_cast<CDrawPin*>(getObject())->which+PINBOX_NORMAL);
  CheckRadioButton(PINBOX_INPUT,PINBOX_PASSIVE,static_cast<CDrawPin*>(getObject())->elec+PINBOX_INPUT);
  CheckDlgButton(PINBOX_NAMEON,  ((static_cast<CDrawPin*>(getObject())->show)&1)!=0);
  CheckDlgButton(PINBOX_NUMBERON,((static_cast<CDrawPin*>(getObject())->show)&2)!=0);

  stop=FALSE;
}



void CEditDlgPinEdit::OnChange()
{
  char str[SIZESTRING];

  if (stop)
	return;

  getObject()->Display();

  // Read in the name
  GetDlgItemText(PINBOX_NAME, str, sizeof(str) );
  static_cast<CDrawPin*>(getObject())->str=str;

  GetDlgItemText(PINBOX_NUMBER, str, sizeof(str) );
  static_cast<CDrawPin*>(getObject())->number=str;

  static_cast<CDrawPin*>(getObject())->show  = IsDlgButtonChecked(PINBOX_NAMEON)!=0 ? 1 : 0;
  static_cast<CDrawPin*>(getObject())->show |= IsDlgButtonChecked(PINBOX_NUMBERON)!=0 ? 2 : 0;

  static_cast<CDrawPin*>(getObject())->dir   = GetCheckedRadioButton(PINBOX_UP,PINBOX_RIGHT)-PINBOX_UP;
  static_cast<CDrawPin*>(getObject())->which = GetCheckedRadioButton(PINBOX_NORMAL,PINBOX_HIDDEN)-PINBOX_NORMAL;
  static_cast<CDrawPin*>(getObject())->elec 	= GetCheckedRadioButton(PINBOX_INPUT,PINBOX_PASSIVE)-PINBOX_INPUT;

  int length = max(5,GetDlgItemInt(PINBOX_LENGTH));
  static_cast<CDrawPin*>(getObject())->length = length;
  getObject()->m_pDesign->GetOptions()->SetPinLength( length );

  getObject()->Display();
}



void CEditDlgPinEdit::ReFocus()
{
  stop = TRUE;

  SetFocus();

  SetDlgItemText(PINBOX_NAME,static_cast<CDrawPin*>(getObject())->str);
  SetDlgItemText(PINBOX_NUMBER,static_cast<CDrawPin*>(getObject())->number);

  stop = FALSE;
}



// The Rotate Dialog

BEGIN_MESSAGE_MAP( CEditDlgRotateBox, CEditDlg )
	ON_BN_CLICKED(ROTATEBLOCK_LEFT, OnLeft)
	ON_BN_CLICKED(ROTATEBLOCK_RIGHT, OnRight)
	ON_BN_CLICKED(ROTATEBLOCK_MIRROR, OnMirror)	
END_MESSAGE_MAP()


// Create this window
void CEditDlgRotateBox::Create()
{
	  CDialog::Create( IDD_ROTATE, &g_EditToolBar );
}

void CEditDlgRotateBox::Open(CTinyCadDoc *pDesign, CDrawingObject *pObject)
{

	Show( pDesign, pObject );
}



void CEditDlgRotateBox::OnLeft()
{
  getObject()->Display();
  static_cast<CDrawBlockRotate*>(getObject())->ChangeDir(3);
  getObject()->Display();
}

void CEditDlgRotateBox::OnRight()
{
  getObject()->Display();
  static_cast<CDrawBlockRotate*>(getObject())->ChangeDir(2);
  getObject()->Display();
}

void CEditDlgRotateBox::OnMirror()
{
  getObject()->Display();
  static_cast<CDrawBlockRotate*>(getObject())->ChangeDir(4);
  getObject()->Display();
}




